@@ -5,11 +5,11 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2016, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2022, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
- * are also available at https://curl.haxx.se/docs/copyright.html.
+ * are also available at https://curl.se/docs/copyright.html.
  *
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
@@ -35,9 +35,10 @@
 
 #define TEST_HANG_TIMEOUT 60 * 1000
 
-#define UPLOADTHIS "this is the blurb we want to upload\n"
+static const char uploadthis[] =
+  "this is the blurb we want to upload\n";
 
-static size_t readcallback(void  *ptr,
+static size_t readcallback(char  *ptr,
                            size_t size,
                            size_t nmemb,
                            void *clientp)
@@ -51,10 +52,10 @@ static size_t readcallback(void  *ptr,
   }
   (*counter)++; /* bump */
 
-  if(size * nmemb > strlen(UPLOADTHIS)) {
+  if(size * nmemb > strlen(uploadthis)) {
     fprintf(stderr, "READ!\n");
-    strcpy(ptr, UPLOADTHIS);
-    return strlen(UPLOADTHIS);
+    strcpy(ptr, uploadthis);
+    return strlen(uploadthis);
   }
   fprintf(stderr, "READ NOT FINE!\n");
   return 0;
@@ -77,9 +78,9 @@ int test(char *URL)
 {
   int res = 0;
   CURL *curl = NULL;
-  int counter=0;
+  int counter = 0;
   CURLM *m = NULL;
-  int running=1;
+  int running = 1;
 
   start_test_timing();
 
@@ -98,13 +99,9 @@ int test(char *URL)
   easy_setopt(curl, CURLOPT_READDATA, &counter);
   /* We CANNOT do the POST fine without setting the size (or choose
      chunked)! */
-  easy_setopt(curl, CURLOPT_POSTFIELDSIZE, (long)strlen(UPLOADTHIS));
+  easy_setopt(curl, CURLOPT_POSTFIELDSIZE, (long)strlen(uploadthis));
 
   easy_setopt(curl, CURLOPT_POST, 1L);
-#ifdef CURL_DOES_CONVERSIONS
-  /* Convert the POST data to ASCII. */
-  easy_setopt(curl, CURLOPT_TRANSFERTEXT, 1L);
-#endif
   easy_setopt(curl, CURLOPT_PROXY, libtest_arg2);
   easy_setopt(curl, CURLOPT_PROXYUSERPWD, libtest_arg3);
   easy_setopt(curl, CURLOPT_PROXYAUTH,
@@ -126,10 +123,6 @@ int test(char *URL)
 
     abort_on_test_timeout();
 
-#ifdef TPF
-    sleep(1); /* avoid ctl-10 dump */
-#endif
-
     if(!running)
       break; /* done */
 
@@ -141,7 +134,7 @@ int test(char *URL)
 
     /* At this point, maxfd is guaranteed to be greater or equal than -1. */
 
-    select_test(maxfd+1, &fdread, &fdwrite, &fdexcep, &timeout);
+    select_test(maxfd + 1, &fdread, &fdwrite, &fdexcep, &timeout);
 
     abort_on_test_timeout();
   }
@@ -157,4 +150,3 @@ test_cleanup:
 
   return res;
 }
-
